@@ -35,23 +35,23 @@
 //! Ideally each instance of concurrent data structure may have its own queue that gets fully
 //! destroyed as soon as the data structure gets dropped.
 
+use alloc::boxed::Box;
 use core::cell::{Cell, UnsafeCell};
 use core::mem::{self, ManuallyDrop};
 use core::num::Wrapping;
 use core::ptr;
 use core::sync::atomic;
 use core::sync::atomic::Ordering;
-use alloc::boxed::Box;
 
-use crossbeam_utils::CachePadded;
 use arrayvec::ArrayVec;
+use crossbeam_utils::CachePadded;
 
 use atomic::Owned;
-use collector::{Handle, Collector};
+use collector::{Collector, LocalHandle};
+use deferred::Deferred;
 use epoch::{AtomicEpoch, Epoch};
 use guard::{unprotected, Guard};
-use deferred::Deferred;
-use sync::list::{List, Entry, IterError, IsElement};
+use sync::list::{Entry, IsElement, IterError, List};
 use sync::queue::Queue;
 
 /// Maximum number of objects a bag can contain.
@@ -184,8 +184,7 @@ impl Global {
             match self.queue.try_pop_if(
                 &|sealed_bag: &SealedBag| sealed_bag.is_expired(global_epoch),
                 guard,
-            )
-            {
+            ) {
                 None => break,
                 Some(sealed_bag) => drop(sealed_bag),
             }
@@ -276,7 +275,7 @@ impl Local {
     const PINNINGS_BETWEEN_COLLECT: usize = 128;
 
     /// Registers a new `Local` in the provided `Global`.
-    pub fn register(collector: &Collector) -> Handle {
+    pub fn register(collector: &Collector) -> LocalHandle {
         unsafe {
             // Since we dereference no pointers in this block, it is safe to use `unprotected`.
 
@@ -290,7 +289,9 @@ impl Local {
                 pin_count: Cell::new(Wrapping(0)),
             }).into_shared(&unprotected());
             collector.global.locals.insert(local, &unprotected());
-            Handle { local: local.as_raw() }
+            LocalHandle {
+                local: local.as_raw(),
+            }
         }
     }
 
@@ -360,10 +361,20 @@ impl Local {
                 //    instruction.
                 //
                 // Both instructions have the effect of a full barrier, but benchmarks have shown
-                // that the second one makes pinning faster in this particular case.
+                // that the second one makes pinning faster in this particular case.  It is not
+                // clear that this is permitted by the C++ memory model (SC fences work very
+                // differently from SC accesses), but experimental evidence suggests that this
+                // works fine.  Using inline assembly would be a viable (and correct) alternative,
+                // but alas, that is not possible on stable Rust.
                 let current = Epoch::starting();
-                let previous = self.epoch.compare_and_swap(current, new_epoch, Ordering::SeqCst);
+                let previous = self
+                    .epoch
+                    .compare_and_swap(current, new_epoch, Ordering::SeqCst);
                 debug_assert_eq!(current, previous, "participant was expected to be unpinned");
+                // We add a compiler fence to make it less likely for LLVM to do something wrong
+                // here.  Formally, this is not enough to get rid of data races; practically,
+                // it should go a long way.
+                atomic::compiler_fence(Ordering::SeqCst);
             } else {
                 self.epoch.store(new_epoch, Ordering::Relaxed);
                 atomic::fence(Ordering::SeqCst);
@@ -406,7 +417,7 @@ impl Local {
         // Update the local epoch only if there's only one guard.
         if guard_count == 1 {
             let epoch = self.epoch.load(Ordering::Relaxed);
-            let global_epoch = self.global().epoch.load(Ordering::Relaxed);
+            let global_epoch = self.global().epoch.load(Ordering::Relaxed).pinned();
 
             // Update the local epoch only if the global epoch is greater than the local epoch.
             if epoch != global_epoch {
@@ -498,14 +509,13 @@ impl IsElement<Local> for Local {
 
 #[cfg(test)]
 mod tests {
-    use std::sync::atomic::{AtomicUsize, ATOMIC_USIZE_INIT};
-    use std::sync::atomic::Ordering;
+    use std::sync::atomic::{AtomicUsize, Ordering};
 
     use super::*;
 
     #[test]
     fn check_defer() {
-        static FLAG: AtomicUsize = ATOMIC_USIZE_INIT;
+        static FLAG: AtomicUsize = AtomicUsize::new(0);
         fn set() {
             FLAG.store(42, Ordering::Relaxed);
         }
@@ -518,7 +528,7 @@ mod tests {
 
     #[test]
     fn check_bag() {
-        static FLAG: AtomicUsize = ATOMIC_USIZE_INIT;
+        static FLAG: AtomicUsize = AtomicUsize::new(0);
         fn incr() {
             FLAG.fetch_add(1, Ordering::Relaxed);
         }
